﻿using SushiBarContracts.BindingModels;
using SushiBarContracts.SearchModels;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;
using SushiBarFileImplement.Models;

namespace SushiBarFileImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataFileSingleton source;
        public OrderStorage()
        {
            source = DataFileSingleton.GetInstance();
        }
        public List<OrderViewModel> GetFullList()
        {
            return source.Orders
                .Select(x => AccessClientStorage(AccessSushiStorage(x.GetViewModel))!)
                .ToList();
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            return source.Orders
                .Where(o => 
                    (!model.Id.HasValue || o.Id == model.Id) &&
                    (!model.DateFrom.HasValue || o.DateCreate >= model.DateFrom) &&
                    (!model.DateTo.HasValue || o.DateCreate <= model.DateTo) &&
                    (!model.ClientId.HasValue || o.ClientId == model.ClientId))
                .Select(o => AccessClientStorage(AccessSushiStorage(o.GetViewModel))!)
                .ToList();
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            return AccessClientStorage(AccessSushiStorage(
                source.Orders
                .FirstOrDefault(x => (x.Id == model.Id))?.GetViewModel));
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            model.Id = source.Orders.Count > 0 ? source.Orders.Max(x => x.Id) + 1 : 1;
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            source.Orders.Add(newOrder);
            source.SaveOrders();
            return AccessClientStorage(AccessSushiStorage(newOrder.GetViewModel));
        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            var element = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (element == null)
            {
                return null;
            }
            element.Update(model);
            source.SaveOrders();
            return AccessClientStorage(AccessSushiStorage(element.GetViewModel));
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            var element = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                source.Orders.Remove(element);
                source.SaveOrders();
                return AccessClientStorage(AccessSushiStorage(element.GetViewModel));
            }
            return null;
        }
        public OrderViewModel? AccessSushiStorage(OrderViewModel? model)
        {
            if (model == null)
                return null;
            foreach (var iceCream in source.Sushis)
            {
                if (iceCream.Id == model.SushiId)
                {
                    model.SushiName = iceCream.SushiName;
                    break;
                }
            }
            return model;
        }

        public OrderViewModel? AccessClientStorage(OrderViewModel? model)
        {
            if (model == null)
                return null;
            var client = source.Clients.FirstOrDefault(x => x.Id == model.Id);
            if (client != null)
                model.ClientFIO = client.ClientFIO;
            return model;
        }
    }
}
